@@ -1,17 +1,18 @@
 ##!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 """
-Created on Fri Mar  1 14:15:56 2019
+Core contains some very basic functions that are used within the package
+in various locations and tools.
 
-@author: and
+@author: Andreas Geiges
 """
 
 import time
-from . import config
+from datatoolbox import config
 import numpy as np
 tt = time.time()
 import pint
-#%% unit handling
+#%% Pint unit handling
 gases = {"CO2eq":"carbon_dioxide_equivalent",
          "CO2e" : "CO2eq",
          "NO2" : "NO2"}
@@ -35,135 +36,25 @@ except pint.errors.DefinitionSyntaxError:
     
 import pint
 
-#%% optional support
-try:
-    import xarray as xr
-    config.AVAILABLE_XARRAY = True
-except:
-    config.AVAILABLE_XARRAY = False   
+#%% optional support xarray
 
+if config.AVAILABLE_XARRAY:
+    from .tools import xarray as _xr
+    to_XDataSet = _xr.to_XDataSet
+    to_XDataArray = _xr.to_XDataArray
 
-def get_dimension_extend(table_iterable, dimensions):
-    """
-    This functions assesses the the unique extend for various dimensions
-    given a set of datatables
-    """
-    fullIdx = dict()
-    for dim in dimensions:
-        fullIdx[dim] = set()
-
-    for table in table_iterable:
-        
-#        for metaKey, metaValue in table.meta.items():
-#            if metaKey not in metaDict.keys():
-#                metaDict[metaKey] = set([metaValue])
-#            else:
-#                metaDict[metaKey].add(metaValue)
-            
-        for dim in dimensions:
-            if dim == 'region':
-                fullIdx[dim] = fullIdx[dim].union(table.index)
-            elif dim == 'time':
-                fullIdx[dim] = fullIdx[dim].union(table.columns)
-            elif dim in table.meta.keys():
-                fullIdx[dim].add(table.meta[dim])
-            else:
-                raise(BaseException('Dimension not available'))
-
-    dimSize = [len(fullIdx[x]) for x in dimensions]
-    dimList = [sorted(list(fullIdx[x])) for x in dimensions]
-    
-    return dimSize, dimList
-    
-
-def get_meta_collection(table_iterable, dimensions):
-
-    
-    metaCollection = dict()
-    for table in table_iterable:
-        
-        for key in table.meta.keys():
-            if key in dimensions  or key == 'ID':
-                continue
-            if key not in metaCollection.keys():
-                metaCollection[key] = set()
-                
-            metaCollection[key].add(table.meta[key])
-    
-    return metaCollection
-
-def to_XDataSet(tableSet, dimensions):
-    
-    dimSize, dimList = get_dimension_extend(tableSet, dimensions= ['region', 'time'])
-    
-    dimensions= ['region', 'time']
-    
-    dSet = xr.Dataset(coords = {key: val for (key, val) in zip(dimensions, dimList)})
-    
-    for key, table in tableSet.items():
-        dSet[key] = table
-        dSet[key].attrs = table.meta
-        
-    return dSet
-    
-def to_XDataArray(tableSet, dimensions = ['region', 'time', 'pathway']):
-    #%%
-#    dimensions = ['region', 'time', 'scenario', 'model']
-    
-#    metaDict = dict()
-    
-    dimSize, dimList = get_dimension_extend(tableSet, dimensions)
-    metaCollection = get_meta_collection(tableSet, dimensions)
-     
-    xData =  xr.DataArray(np.zeros(dimSize)*np.nan, coords=dimList, dims=dimensions)
-    
-    for table in tableSet:
-        
-        indexTuple = list()
-        for dim in dimensions:
-            if dim == 'region':
-                indexTuple.append(list(table.index))
-            elif dim == 'time':
-                indexTuple.append(list(table.columns))
-            else:
-                indexTuple.append(table.meta[dim])
-                
-#        xx = (table.index,table.columns,table.meta['pathway'])
-        xData.loc[tuple(indexTuple)] = table.values
-        
-        
-    # only implemented for homgeneous physical units
-    assert len(metaCollection['unit']) == 1
-    xData.attrs['unit'] = list(metaCollection['unit'])[0]   
-    
-    for  metaKey, metaValue in metaCollection.items():
-        if len(metaValue) == 1:
-            xData.attrs[metaKey] = metaValue.pop()
-        else:
-            print('Warning, dropping meta data: ' + metaKey +  ' ' + str(metaValue))
-    
-    return xData
 #%%
 
 c = pint.Context('GWP_AR5')
 
 CO2EQ_LIST = ['CO2eq', 'CO2e',]
 
-AR4GWPDict = {'F-Gases': 1,
-              'Sulfur' : 1,
-              'CH4': 25,
+AR4GWPDict = {'CH4': 25,
               'HFC': 1430,
               'N2O' : 298,
-              'NH3' : 1,
-              'NOx' : 1,
-              'VOC' : 1 ,
               'SF6' : 22800,
               'PFC' : 7390,
-              'CO2' : 1,
-              'BC'  :1,
-              'CO' : 1,
-              'OC' : 1,
-              'SO2' : 1}
+              'CO2' : 1}
 
 
 LOG = dict()
@@ -172,6 +63,20 @@ LOG['tableIDs'] = list()
 ur.add_context(c)
 
 def _update_meta(metaDict):
+    """
+    Private funcion to update the meta data of a datatable
+    
+
+    Parameters
+    ----------
+    metaDict : dict
+        new data to overwrite.
+
+    Returns
+    -------
+    metaDict : dict
+
+    """
     
     for key in list(metaDict.keys()):
         if (metaDict[key] is np.nan) or metaDict[key] == '':
@@ -191,12 +96,35 @@ def _createDatabaseID(metaDict):
 
 
 def osIsWindows():
+    """
+    Checkes if operating system is windows based
+    
+
+    Returns
+    -------
+    bool
+
+    """
     if (config.OS == 'win32') | (config.OS == "Windows"):
         return True
     else:
         return False
 
 def getUnit(string):
+    """
+    Return the pint unit for a given unit string. Compared to the original
+    pint this functions replaces special characters $ € and % by a string 
+    reprentation.
+
+    Parameters
+    ----------
+    string : str
+        unit string (e.g. "km / s" or "€  / capita")
+
+    Returns
+    -------
+    unit : pint unit
+    """
     if string is None:
         string = ''
     else:
@@ -204,6 +132,19 @@ def getUnit(string):
     return ur(string)
 
 def getUnitWindows(string):
+    """
+    Equivalent version of getUnit  but adapted for windows system.
+    
+    Parameters
+    ----------
+    string : str
+        unit string (e.g. "km / s" or "€  / capita")
+
+    Returns
+    -------
+    unit : pint unit
+
+    """
     if string is None:
         string = ''
     else:
@@ -216,13 +157,47 @@ if osIsWindows():
 
 
 def getTimeString():
+    """
+    Return formated time string.
+
+    Returns
+    -------
+    time string : str
+
+    """
     return time.strftime("%Y/%m/%d-%I:%M:%S")
 
 def getDateString():
+    """
+    Return formated date string.
+
+    Returns
+    -------
+    date string : str
+
+    """
     return time.strftime("%Y/%m/%d")
 
 def conversionFactor(unitFrom, unitTo, context =None):
-    
+    """
+    Return the conversion factor from one unit to another.
+
+    Parameters
+    ----------
+    unitFrom : str
+        Original unit to convert.
+    unitTo : str
+        Unit to which it original unit should be converted.
+    context : str, optional
+        For some conversions, a specifice context is needed. Currently, only
+        GWPAR4 is implemented. The default is None.
+
+    Returns
+    -------
+    TYPE
+        DESCRIPTION.
+
+    """ 
     if context is None:
         return getUnit(unitFrom).to(getUnit(unitTo)).m
     elif context == 'GWPAR4':
@@ -261,6 +236,71 @@ def _AR4_conversionFactor(unitFrom, unitTo):
     co2eq_factor = AR4GWPDict[gasKey]
     factor = conversFactor * co2eq_factor
     return factor
+
+
+def get_dimension_extend(table_iterable, dimensions):
+    """
+    This functions assesses the the unique extend for various dimensions
+    given a set of datatables
+    """
+    fullIdx = dict()
+    for dim in dimensions:
+        fullIdx[dim] = set()
+
+    for table in table_iterable:
+        
+#        for metaKey, metaValue in table.meta.items():
+#            if metaKey not in metaDict.keys():
+#                metaDict[metaKey] = set([metaValue])
+#            else:
+#                metaDict[metaKey].add(metaValue)
+            
+        for dim in dimensions:
+            if dim == 'region':
+                fullIdx[dim] = fullIdx[dim].union(table.index)
+            elif dim == 'time':
+                fullIdx[dim] = fullIdx[dim].union(table.columns)
+            elif dim in table.meta.keys():
+                fullIdx[dim].add(table.meta[dim])
+            else:
+                raise(BaseException('Dimension not available'))
+
+    dimSize = [len(fullIdx[x]) for x in dimensions]
+    dimList = [sorted(list(fullIdx[x])) for x in dimensions]
+    
+    return dimSize, dimList
+    
+
+def get_meta_collection(table_iterable, dimensions):
+    """
+
+    Parameters
+    ----------
+    table_iterable : list of tables
+        DESCRIPTION.
+    dimensions : list of dimentions
+        DESCRIPTION.
+
+    Returns
+    -------
+    metaCollection : TYPE
+        DESCRIPTION.
+
+    """
+
+    
+    metaCollection = dict()
+    for table in table_iterable:
+        
+        for key in table.meta.keys():
+            if key in dimensions  or key == 'ID':
+                continue
+            if key not in metaCollection.keys():
+                metaCollection[key] = set()
+                
+            metaCollection[key].add(table.meta[key])
+    
+    return metaCollection
     
 if config.DEBUG:
     print('core loaded in {:2.4f} seconds'.format(time.time()-tt))
